
/* ImpEXSIDE_PS - Double band-selective EXSIDE to measure long-range
   coupling constants with IMPRESS and real-time PureShift acquisition options

        Parameters:
                sspul :         selects magnetization randomization option
                nullflg:        selects TANGO-Gradient option
                hsglvl:         Homospoil gradient level (DAC units)
                hsgt    :       Homospoil gradient time
                gzlvlE  :       encoding Gradient level
                gtE     :       encoding gradient time
                EDratio :       Encode/Decode ratio
                gstab   :       recovery delay
                j1xh    :       One-bond XH coupling constant
                impress :       Selects impress option
                                0 - sets non-impress bandselective option
                                  uses selpwxA,selpwxlvlA,selpwxshpA for shape1
                                       selpwxB,selpwxlvlB,selpwxshpB for shape2
                                non-zero - Number bands in impress
                imphase :       1,2 - 2 band IMPRESS
                                1,2,3,4 - 3 or 4 band IMPRESS
                                1-4 uses imppw1, imppwr1,impshp1 for shape1
                                1 - uses imppw1,imppwr1,impshp1 for shape2
                                2 - uses imppw2,imppwr2,impshp2 for shape2
                                3 - uses imppw3,imppwr3,impshp3 for shape2
                                4 - user imppw4,imppwr4,impshp4 for shape2

                HOBS	   :  'y' - real time PureShift acquisition
			      'n' - impicit acquition, no decoupling
		npoints    :  number of data points acquired in a chunk
                                  (np must be an integer multiple of npoints)
		chunkwidth :  lenght of a single chunk
                tauD       :  effect of alfa
                gtcr       :  crusher gradient duration
                gzlvlcr    :  crusher gradient amplitude
                nt         :  number of transient per increment
                              2 recommended minimun
                              4 for proper F1 quad-image cancellation

Krish Krishnamurthy - Aug. 95 (original EXSIDE)

[REF: V.V. Krishnamurthy, JMR, A121, 33-41 (1996)]

Krish Krishnamurthy - Oct. 2004 (revised)

Craig Butts, Univ. Bristol, UK, Bert Heise, Agilent UK - Feb. 2012
  (double band-selective, IMPRESS and Pure Shift implementation)

Ikenna Ndukwe, University of Bristol - Sept. 2014  (IMPRESS and Pure Shift implementation)

*/

#include <standard.h>
#include <chempack.h>
#ifndef   FIRST_FID
#include <Pbox_psg.h>
#endif

static int      ph4[8]  =  {0,0,0,0,2,2,2,2},
                ph2[2]  =  {0,2},
                ph1[4]  =  {0,0,2,2},
                ph5[8]  =  {0,2,2,0,2,0,0,2};

static shape rtsh;

pulsesequence()

{

   double   selpwrA = getval("selpwrA"),
            selpwA = getval("selpwA"),
            gzlvlA = getval("gzlvlA"),
            gtA = getval("gtA"),
            selpwrB = getval("selpwrB"),
            selpwB = getval("selpwB"),
            gzlvlB = getval("gzlvlB"),
            gtB = getval("gtB"),
   	    gzlvlE = getval("gzlvlE"),
            gtE = getval("gtE"),
            EDratio = getval("EDratio"),
            gstab = getval("gstab"),
	    pwx180 = getval("pwx180"),
	    pwxlvl180 = getval("pwxlvl180"),
            hsglvl = getval("hsglvl"),
            hsgt = getval("hsgt"),
            selpwxA = getval("selpwxA"),
            selpwxlvlA = getval("selpwxlvlA"),
            selpwxB = getval("selpwxB"),
            selpwxlvlB = getval("selpwxlvlB"),
	    impress = getval("impress"),
            tauA=getval("tauA"),        //compensation for tauB and tauD
            tauB=getval("tauB"),        //effect of rof2
            tauD=getval("tauD"),        //effect of alfa
            gzlvlcr = getval("gzlvlcr"),
            gzlvlAcr = getval("gzlvlAcr"),
            gzlvlBcr = getval("gzlvlBcr"),
            gtcr = getval("gtcr"),        //crusher gradient in HOBS
            gtAcr = getval("gtAcr"),      //crusher gradient in HOBS
            gtBcr = getval("gtBcr"),
            npoints = getval("npoints"),  // npoints should be an integer multiple of np
		bw_rt = getval("bw_rt"),
		of_rt = getval("of_rt"),
		bw_rt1 = getval("bw_rt1"),
		of_rt1 = getval("of_rt1"),
            tau,
	    evolcorr,
	    jdly,
	    bird,cycles;
   int      icosel,
	    ijscale,
            prgcycle = (int)(getval("prgcycle")+0.5),
	    phase1=(int)(getval("phase")+0.5),
            iimphase = (int)(getval("imphase")+0.5);
   char     selshapeA[MAXSTR],
            selshapeB[MAXSTR],
   	    pwx180ad[MAXSTR],
	    pwx180adR[MAXSTR],
            selpwxshpA[MAXSTR],
            selpwxshpB[MAXSTR],
            HOBS[MAXSTR],
            rtsel[MAXSTR],
            cmd[MAXSTR];

//synchronize gradients to srate for probetype='nano'
//   Preserve gradient "area"
  gtE = syncGradTime("gtE","gzlvlE",0.5);
  gzlvlE = syncGradLvl("gtE","gzlvlE",0.5);
  gtA = syncGradTime("gtA","gzlvlA",1.0);
  gzlvlA = syncGradLvl("gtA","gzlvlA",1.0);
  gtB = syncGradTime("gtB","gzlvlB",1.0);
  gzlvlB = syncGradLvl("gtB","gzlvlB",1.0);

  getstr("selshapeA",selshapeA);
  getstr("selshapeB",selshapeB);
  getstr("pwx180ad", pwx180ad);
  getstr("pwx180adR", pwx180adR);
  getstr("selpwxshpA", selpwxshpA);
  getstr("selpwxshpB", selpwxshpB);
  getstr("HOBS",HOBS);
  getstr("rtsel",rtsel);

  evolcorr=(4*pwx/PI)+2*pw+8.0e-6;
  bird = 1 / (4*(getval("j1xh")));
  tau = 1/(4*(getval("jnxh")));
  ijscale=(int)(getval("jscale") + 0.5);
  jdly=(d2*ijscale);
  cycles=np/npoints;

     cycles = (double)((int)((cycles)));
     initval(cycles,v20);

  icosel = 1;

  if (impress > 0.5)
  {
    selpwxA = getval("imppw1");
    selpwxlvlA = getval("imppwr1");
    getstr("impshp1", selpwxshpA);
    if (iimphase == 1)
    {
        selpwxB = getval("imppw1");
        selpwxlvlB = getval("imppwr1");
        getstr("impshp1",selpwxshpB);
    }
    if (iimphase == 2)
    {
        selpwxB = getval("imppw2");
        selpwxlvlB = getval("imppwr2");
        getstr("impshp2",selpwxshpB);
    }
    if (iimphase == 3)
    {
        selpwxB = getval("imppw3");
        selpwxlvlB = getval("imppwr3");
        getstr("impshp3",selpwxshpB);
    }
    if (iimphase == 4)
    {
        selpwxB = getval("imppw4");
        selpwxlvlB = getval("imppwr4");
        getstr("impshp4",selpwxshpB);
    }
  }

  assign(ct,v17);
  assign(zero,v18);
  assign(zero,v19);

//generation of rt shaped pulse

  if (rtsel[0] == 'y')
  {
     if (FIRST_FID)
     {
        sprintf(cmd, "Pbox rtEXSIDEshape -u %s -w \"rsnob %f %f\" -p %.0f -l %.2f -s 2.0",userdir,bw_rt1,of_rt1,tpwr, 1.0e6*pw);
        system(cmd);
        rtsh= getRsh("rtEXSIDEshape");
//        printf("%f\n %f\n %f\n",rtsh.pwr,rtsh.pw,rtsh.pwrf);
     }
  }
  else
  {
     if (FIRST_FID)
     {
        sprintf(cmd, "Pbox rtEXSIDEshape -u %s -w \"rsnob %f %f\" -p %.0f -l %.2f -s 2.0",userdir,bw_rt,of_rt,tpwr, 1.0e6*pw);
        system(cmd);
        rtsh= getRsh("rtEXSIDEshape");
//        printf("%f\n %f\n %f\n",rtsh.pwr,rtsh.pw,rtsh.pwrf);
     }
  }

  if (getflag("prgflg") && (satmode[0] == 'y') && (prgcycle > 1.5))
    {
        hlv(ct,v17);
        mod2(ct,v18); dbl(v18,v18);
        if (prgcycle > 2.5)
           {
                hlv(v17,v17);
                hlv(ct,v19); mod2(v19,v19); dbl(v19,v19);
           }
     }

   settable(t1,4,ph1);
   settable(t2,2,ph2);
   settable(t4,8,ph4);
   settable(t5,8,ph5);

   getelem(t1,v17,v1);
   getelem(t2,v17,v2);
   getelem(t4,v17,v4);
   getelem(t5,v17,oph);

  add(oph,v18,oph);
  add(oph,v19,oph);

  if ((phase1 == 2) || (phase1 == 5))
    icosel = -1;

  initval(2.0*(double)(((int)(d2*getval("sw1")+0.5)%2)),v14);
/*
  mod2(id2, v14);
  dbl(v14,v14);
*/

  if (impress > 0.5)
  {
	if (getflag("fadflg"))
	{
  		add(v2, v14, v2);
  		add(oph, v14, oph);
	}
  }
  else
        {
                add(v2, v14, v2);
                add(oph, v14, oph);
        }


status(A);
   obspower(tpwr);

   delay(5.0e-5);
   if (getflag("sspul"))
        steadystate();
   if (satmode[0] == 'y')
     {
        if ((d1-satdly) > 0.02)
                delay(d1-satdly);
        else
                delay(0.02);
        if (getflag("slpsat"))
           {
                shaped_satpulse("relaxD",satdly,zero);
                if (getflag("prgflg"))
                   shaped_purge(v1,zero,v18,v19);
           }
        else
           {
                satpulse(satdly,zero,rof1,rof1);
                if (getflag("prgflg"))
                   purge(v1,zero,v18,v19);
           }
     }
   else
        delay(d1);

   if (getflag("wet"))
     wet4(zero,one);

   decpower(pwxlvl);

  status(B);

    if (getflag("nullflg"))
    {
      rgpulse(0.5 * pw, zero, rof1, rof1);
      delay(2 * bird);
      decpower(pwxlvl180);
      decshaped_pulse(pwx180ad, pwx180, zero, rof1, rof1);
      rgpulse(2.0 * pw, zero, rof1, rof1);
      delay(2 * bird + 2 * POWER_DELAY);
      decshaped_pulse(pwx180adR, pwx180, zero, rof1, rof1);
      decpower(pwxlvl);
      rgpulse(1.5 * pw, zero, rof1, rof1);
      zgradpulse(hsglvl, hsgt);
      delay(gstab);
    }

    rgpulse(pw, v1, rof1, rof1);
    delay(tau/2 - gtA - gstab);
    delay(jdly/4);

    if (getflag("impflg"))
    {
        zgradpulse(gzlvlA,gtA);
        delay(gstab);
        obspower(selpwrA);
        decpower(pwxlvl180);
    simshaped_pulse(selshapeA,pwx180ad,selpwA,pwx180,zero,zero,rof1,rof1);
        zgradpulse(gzlvlA,gtA);
        delay(gstab);

    delay(tau/2 - gtA - gstab);
    delay(jdly/2);
    delay(tau/2 - gtB - gstab);

        zgradpulse(gzlvlB,gtB);
        delay(gstab);
        obspower(selpwrB);
    simshaped_pulse(selshapeB,pwx180adR,selpwB,pwx180,two,zero,rof1,rof1);
	obspower(tpwr);
        zgradpulse(gzlvlB,gtB);
        delay(gstab);
    }
    else
    {
        zgradpulse(gzlvlA,gtA);
        delay(gstab);
        obspower(selpwrA);
        decpower(selpwxlvlA);
    simshaped_pulse(selshapeA,selpwxshpA,selpwA,selpwxA,zero,zero,rof1,rof1);
        zgradpulse(gzlvlA,gtA);
        delay(gstab);

    delay(tau/2 - gtA - gstab);
    delay(jdly/2);
    delay(tau/2 - gtB - gstab);

        zgradpulse(gzlvlB,gtB);
        delay(gstab);
        obspower(selpwrB);

        decpower(selpwxlvlB);
    simshaped_pulse(selshapeB,selpwxshpB,selpwB,selpwxB,two,zero,rof1,rof1);
        obspower(tpwr);
        zgradpulse(gzlvlB,gtB);
        delay(gstab);
    }
	decpower(pwxlvl);

    delay(jdly/4);
    delay(tau/2 - gtB - gstab);

    rgpulse(pw, one, rof1, rof1);
    zgradpulse(hsglvl, 2 * hsgt);
    delay(gstab);
    decrgpulse(pwx, v2, rof1, 2.0e-6);

      delay(d2/2);
    rgpulse(2 * pw, zero, 2.0e-6, 2.0e-6);
      delay(d2/2);

    if (getflag("impflg"))
    {
        zgradpulse(gzlvlA,gtA);
        delay(gstab);
        decpower(selpwxlvlA);
        decshaped_pulse(selpwxshpA, selpwxA, zero, rof1, rof1);
        decpower(pwxlvl);
        zgradpulse(gzlvlA,gtA);
        delay(gstab + evolcorr);

        zgradpulse(gzlvlE, gtE/2);
        delay(100e-6);

        zgradpulse(gzlvlB,gtB);
        delay(gstab);
        decpower(selpwxlvlB);
        decshaped_pulse(selpwxshpB, selpwxB, zero, rof1, rof1);
        decpower(pwxlvl);
        zgradpulse(gzlvlB,gtB);
        delay(100e-6);

        zgradpulse(-gzlvlE, gtE/2);
        delay(gstab);
    }
    else
    {
        decpower(pwxlvl180);
        decshaped_pulse(pwx180ad, pwx180, zero, rof1, rof1);
        zgradpulse(gzlvlE, gtE);
        delay(gstab + POWER_DELAY + evolcorr);
        decshaped_pulse(pwx180ad, pwx180, zero, rof1, rof1);
        decpower(pwxlvl);
    }

    decrgpulse(pwx, v4, 2.0e-6, rof1);
    zgradpulse(-0.6 * hsglvl, 1.2 * hsgt);
    delay(gstab);
    rgpulse(pw, zero, rof1, rof1);

    zgradpulse(icosel * 2.0*gzlvlE/EDratio, gtE/2.0);
    delay(gstab);

    delay(tau - gtA - 2*gstab - gtE/2);
        zgradpulse(gzlvlA,gtA);
        delay(gstab);
        obspower(selpwrA);
        decpower(pwxlvl180);
        decshaped_pulse(pwx180ad, pwx180, zero, rof1, rof1);
        shaped_pulse(selshapeA,selpwA,zero,rof1,rof1);
        zgradpulse(gzlvlA,gtA);
        delay(gstab);

    delay(tau - gtA - gstab);

        decshaped_pulse(pwx180adR, pwx180, zero, rof1, rof1);
        decpower(dpwr);
        zgradpulse(gzlvlB,gtB);
        delay(gstab);
        obspower(selpwrB);
        shaped_pulse(selshapeB,selpwB,two,rof1,rof2);
        obspower(tpwr);
        zgradpulse(gzlvlB,gtB);
        delay(gstab);

/********Pure shift HOBS Acquisition starts here*************/

        if (HOBS[0]=='y')
        {
                setacqmode(WACQ|NZ);
                obsblank();
                startacq(alfa);
        }

/*----------------------------------------------------------------------------
         Observe the 1st half chunk
-----------------------------------------------------------------------------*/

        if (HOBS[0]=='y')
        {
                status(C);
                acquire(npoints/2.0,1.0/sw);
                rcvroff();                                            

                status(B);
                obspower(tpwr);
                zgradpulse(gzlvlcr,gtcr);
                delay(gstab);
                rgpulse(2.0*pw,zero,rof1,rof1);
                zgradpulse(gzlvlcr,gtcr);
                delay(gstab);
                delay(tauA);
                zgradpulse(gzlvlAcr,gtAcr);
                delay(gstab);
                obspower(rtsh.pwr);
                shaped_pulse(rtsh.name,rtsh.pw,zero,rof1,rof1);
                zgradpulse(gzlvlAcr,gtAcr);
                delay(gstab);
                obsblank();
                delay(tauB);
                rcvron();       //this includes rof3
                delay(tauD);
                decr(v20);

/*------------------------------------------------------------------------------
                Loops for more chunks
------------------------------------------------------------------------------*/

                starthardloop(v20);

                status(C);
                acquire(npoints,1.0/sw);
                rcvroff();                                            

                status(B);
                obspower(tpwr);
                zgradpulse(gzlvlcr,gtcr);
                delay(gstab);
                rgpulse(2.0*pw,zero,rof1,rof1);
                zgradpulse(gzlvlcr,gtcr);
                delay(gstab);
                delay(tauA);
                zgradpulse(gzlvlAcr,gtAcr);
                delay(gstab);
                obspower(rtsh.pwr);
                shaped_pulse(rtsh.name,rtsh.pw,zero,rof1,rof1);
                zgradpulse(gzlvlAcr,gtAcr);
                delay(gstab);
                obsblank();
                delay(tauB);
                rcvron();       //this includes rof3
                delay(tauD);

                endhardloop();

/*----------------------------------------------------------------
                Acquisition of last half chunk
----------------------------------------------------------------*/
                status(C);
                acquire(npoints/2.0,1.0/sw);
                rcvroff();                                    
                endacq();
                incr(v20);
        }
                 /****** HOBS ends here for all *****/
/***************** ACQ for conventional ImpEXSIDE/bsEXSIDE ******************************/
        else
        status(C);
}
/**************** PULSE SEQUENCE ENDS HERE**********************************/

